@@ -52,6 +52,7 @@
 #include "gedit-document.h"
 #include "gedit-prefs-manager.h"
 #include "gedit-debug.h"
+#include "gedit-convert.h"
 
 #define STDIN_DELAY_MICROSECONDS 100000
 
@@ -319,81 +320,6 @@ gedit_dialog_add_button (GtkDialog *dialog, const gchar* text, const gchar* stoc
 	return button;
 }
 
-/*
- * ATTENTION:
- * THIS IS AN ATTEMPT TO MAKE A FULLY INTERNATIONALIZED
- * CASELESS SEARCH
- * IT STILL HAS SOME BUGS
- */
-
-static gchar *g_utf8_strcasestr(const gchar *haystack, const gchar *needle);
-
-static gchar *
-g_utf8_strcasestr(const gchar *haystack, const gchar *needle)
-{
-	gsize needle_len;
-	gsize haystack_len;
-	gchar *ret = NULL;
-	gchar *p;
-	gchar *casefold;
-	gchar *caseless_haystack;
-	gchar *caseless_needle;
-	
-	g_return_val_if_fail (haystack != NULL, NULL);
-	g_return_val_if_fail (needle != NULL, NULL);
-	      
-	casefold = g_utf8_casefold (haystack, -1);
-	caseless_haystack = g_utf8_normalize (casefold, -1, G_NORMALIZE_ALL);
-	g_free (casefold);
-
-	casefold = g_utf8_casefold (needle, -1);
-	caseless_needle = g_utf8_normalize (casefold, -1, G_NORMALIZE_ALL);
-	g_free (casefold);
-
-	needle_len = g_utf8_strlen (caseless_needle, -1);
-	haystack_len = g_utf8_strlen (caseless_haystack, -1);
-
-	if (needle_len == 0)
-	{
-		ret = (gchar *)haystack;
-		goto finally_1;
-	}
-
-      	if (haystack_len < needle_len)
-	{
-		ret = NULL;
-		goto finally_1;
-	}
-			
-	p = (gchar*)haystack;
-	needle_len = strlen (caseless_needle);
-	
-	/* Not so efficient */
-	while (*p)
-	{
-		if ((strncmp (caseless_haystack, caseless_needle, needle_len) == 0))
-		{
-			ret = p;
-			goto finally_1;
-		}
-
-		g_free (caseless_haystack);
-					
-		p = g_utf8_next_char (p);	
-
-		casefold = g_utf8_casefold (p, -1);
-		caseless_haystack = g_utf8_normalize (casefold, -1, G_NORMALIZE_ALL);
-		g_free (casefold);
-	}      
-
-finally_1:
-
-	g_free (caseless_needle);
-	g_free (caseless_haystack);
-	
-	return ret;
-}
-
 /*
  * n: len of the string in bytes
  */
@@ -435,324 +361,6 @@ finally_2:
 	return ret;
 }
 
-/**********************************************************
- * The following code is a modified version of the gtk one
- **********************************************************/
-
-/*	
-gboolean      _gtk_text_btree_char_is_invisible (const GtkTextIter *iter);
-*/
-#define GTK_TEXT_UNKNOWN_CHAR 0xFFFC
-
-
-static void
-forward_chars_with_skipping (GtkTextIter *iter,
-                             gint         count,
-                             gboolean     skip_invisible,
-                             gboolean     skip_nontext)
-{
-
-  gint i;
-
-  g_return_if_fail (count >= 0);
-
-  i = count;
-
-  while (i > 0)
-    {
-      gboolean ignored = FALSE;
-
-      if (skip_nontext &&
-          gtk_text_iter_get_char (iter) == GTK_TEXT_UNKNOWN_CHAR)
-        ignored = TRUE;
-
-      if (!ignored &&
-          skip_invisible &&
-          /* _gtk_text_btree_char_is_invisible (iter)*/ FALSE)
-        ignored = TRUE;
-
-      gtk_text_iter_forward_char (iter);
-
-      if (!ignored)
-        --i;
-    }
-}
-
-static gboolean
-lines_match (const GtkTextIter *start,
-             const gchar **lines,
-             gboolean visible_only,
-             gboolean slice,
-             GtkTextIter *match_start,
-             GtkTextIter *match_end)
-{
-  GtkTextIter next;
-  gchar *line_text;
-  const gchar *found;
-  gint offset;
-
-  if (*lines == NULL || **lines == '\0')
-    {
-      if (match_start)
-        *match_start = *start;
-
-      if (match_end)
-        *match_end = *start;
-      return TRUE;
-    }
-
-  next = *start;
-  gtk_text_iter_forward_line (&next);
-
-  /* No more text in buffer, but *lines is nonempty */
-  if (gtk_text_iter_equal (start, &next))
-    {
-      return FALSE;
-    }
-
-  if (slice)
-    {
-      if (visible_only)
-        line_text = gtk_text_iter_get_visible_slice (start, &next);
-      else
-        line_text = gtk_text_iter_get_slice (start, &next);
-    }
-  else
-    {
-      if (visible_only)
-        line_text = gtk_text_iter_get_visible_text (start, &next);
-      else
-        line_text = gtk_text_iter_get_text (start, &next);
-    }
-
-  if (match_start) /* if this is the first line we're matching */
-    found = g_utf8_strcasestr (line_text, *lines);
-  else
-    {
-      /* If it's not the first line, we have to match from the
-       * start of the line.
-       */
-      if (g_utf8_caselessnmatch (line_text, *lines, strlen (line_text), strlen (*lines)))
-        found = line_text;
-      else
-        found = NULL;
-    }
-
-  if (found == NULL)
-    {
-      g_free (line_text);
-      return FALSE;
-    }
-
-  /* Get offset to start of search string */
-  offset = g_utf8_strlen (line_text, found - line_text);
-
-  next = *start;
-
-  /* If match start needs to be returned, set it to the
-   * start of the search string.
-   */
-  if (match_start)
-    {
-      *match_start = next;
-
-      forward_chars_with_skipping (match_start, offset,
-                                   visible_only, !slice);
-    }
-
-  /* Go to end of search string */
-  offset += g_utf8_strlen (*lines, -1);
-
-  forward_chars_with_skipping (&next, offset,
-                               visible_only, !slice);
-
-  g_free (line_text);
-
-  ++lines;
-
-  if (match_end)
-    *match_end = next;
-
-  /* pass NULL for match_start, since we don't need to find the
-   * start again.
-   */
-  return lines_match (&next, lines, visible_only, slice, NULL, match_end);
-}
-/* strsplit () that retains the delimiter as part of the string. */
-static gchar **
-strbreakup (const char *string,
-            const char *delimiter,
-            gint        max_tokens)
-{
-  GSList *string_list = NULL, *slist;
-  gchar **str_array, *s;
-  guint i, n = 1;
-
-  g_return_val_if_fail (string != NULL, NULL);
-  g_return_val_if_fail (delimiter != NULL, NULL);
-
-  if (max_tokens < 1)
-    max_tokens = G_MAXINT;
-
-  s = strstr (string, delimiter);
-  if (s)
-    {
-      guint delimiter_len = strlen (delimiter);
-
-      do
-        {
-          guint len;
-          gchar *new_string;
-
-          len = s - string + delimiter_len;
-          new_string = g_new (gchar, len + 1);
-          strncpy (new_string, string, len);
-          new_string[len] = 0;
-          string_list = g_slist_prepend (string_list, new_string);
-          n++;
-          string = s + delimiter_len;
-          s = strstr (string, delimiter);
-        }
-      while (--max_tokens && s);
-    }
-  if (*string)
-    {
-      n++;
-      string_list = g_slist_prepend (string_list, g_strdup (string));
-    }
-
-  str_array = g_new (gchar*, n);
-
-  i = n - 1;
-
-  str_array[i--] = NULL;
-  for (slist = string_list; slist; slist = slist->next)
-    str_array[i--] = slist->data;
-
-  g_slist_free (string_list);
-
-  return str_array;
-}
-
-/**
- * gedit_text_iter_forward_search:
- * @iter: start of search
- * @str: a search string
- * @flags: flags affecting how the search is done
- * @match_start: return location for start of match, or %NULL
- * @match_end: return location for end of match, or %NULL
- * @limit: bound for the search, or %NULL for the end of the buffer
- * 
- * Searches forward for @str. Any match is returned as the range
- * @match_start, @match_end. The search will not continue past
- * @limit. Note that a search is a linear or O(n) operation, so you
- * may wish to use @limit to avoid locking up your UI on large
- * buffers.
- * 
- * If the #GTK_TEXT_SEARCH_VISIBLE_ONLY flag is present, the match may
- * have invisible text interspersed in @str. i.e. @str will be a
- * possibly-noncontiguous subsequence of the matched range. similarly,
- * if you specify #GTK_TEXT_SEARCH_TEXT_ONLY, the match may have
- * pixbufs or child widgets mixed inside the matched range. If these
- * flags are not given, the match must be exact; the special 0xFFFC
- * character in @str will match embedded pixbufs or child widgets.
- *
- * Return value: whether a match was found
- **/
-gboolean
-gedit_text_iter_forward_search (const GtkTextIter *iter,
-                              const gchar       *str,
-                              GtkTextSearchFlags flags,
-                              GtkTextIter       *match_start,
-                              GtkTextIter       *match_end,
-                              const GtkTextIter *limit)
-{
-  gchar **lines = NULL;
-  GtkTextIter match;
-  gboolean retval = FALSE;
-  GtkTextIter search;
-  gboolean visible_only;
-  gboolean slice;
-  
-  g_return_val_if_fail (iter != NULL, FALSE);
-  g_return_val_if_fail (str != NULL, FALSE);
-
-  if ((flags & GTK_TEXT_SEARCH_CASE_INSENSITIVE) == 0)
-	  return gtk_text_iter_forward_search (iter, str, flags,
-			 match_start, match_end, limit); 
-
-  if (limit &&
-      gtk_text_iter_compare (iter, limit) >= 0)
-    return FALSE;
-  
-  if (*str == '\0')
-    {
-      /* If we can move one char, return the empty string there */
-      match = *iter;
-      
-      if (gtk_text_iter_forward_char (&match))
-        {
-          if (limit &&
-              gtk_text_iter_equal (&match, limit))
-            return FALSE;
-          
-          if (match_start)
-            *match_start = match;
-          if (match_end)
-            *match_end = match;
-          return TRUE;
-        }
-      else
-        return FALSE;
-    }
-
-  visible_only = (flags & GTK_TEXT_SEARCH_VISIBLE_ONLY) != 0;
-  slice = (flags & GTK_TEXT_SEARCH_TEXT_ONLY) == 0;
-  
-  /* locate all lines */
-
-  lines = strbreakup (str, "\n", -1);
-
-  search = *iter;
-
-  do
-    {
-      /* This loop has an inefficient worst-case, where
-       * gtk_text_iter_get_text () is called repeatedly on
-       * a single line.
-       */
-      GtkTextIter end;
-
-      if (limit &&
-          gtk_text_iter_compare (&search, limit) >= 0)
-        break;
-      
-      if (lines_match (&search, (const gchar**)lines,
-                       visible_only, slice, &match, &end))
-        {
-          if (limit == NULL ||
-              (limit &&
-               gtk_text_iter_compare (&end, limit) < 0))
-            {
-              retval = TRUE;
-              
-              if (match_start)
-                *match_start = match;
-              
-              if (match_end)
-	        *match_end = end;
-	      
-            }
-          
-          break;
-        }
-    }
-  while (gtk_text_iter_forward_line (&search));
-
-  g_strfreev ((gchar**)lines);
-
-  return retval;
-}
 
 /*************************************************************
  * ERROR REPORTING CODE
@@ -763,13 +371,15 @@ gedit_text_iter_forward_search (const GtkTextIter *iter,
 void
 gedit_utils_error_reporting_loading_file (
 		const gchar *uri,
+		const GeditEncoding *encoding,
 		GError *error,
 		GtkWindow *parent)
 {
 	gchar *scheme_string;
-	gchar *error_message;
+	gchar *error_message = NULL;
 	gchar *full_formatted_uri;
        	gchar *uri_for_display	;
+	gchar *encoding_name = NULL;
 	
 	GnomeVFSURI *vfs_uri;
 	
@@ -788,194 +398,278 @@ gedit_utils_error_reporting_loading_file (
 			MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 
-	switch (error->code)
+	if (encoding != NULL)
+		encoding_name = gedit_encoding_to_string (encoding);
+	else
+		encoding_name = g_strdup ("UTF-8");
+	
+	if (error->domain == GEDIT_DOCUMENT_IO_ERROR)
 	{
-		case GNOME_VFS_ERROR_NOT_FOUND:
-			error_message = g_strdup_printf (
-                        	_("Could not find the file \"%s\".\n\n"
-			   	  "Please, check that you typed the location correctly and try again."),
-                         	uri_for_display);
-			break;
-
-		case GNOME_VFS_ERROR_CORRUPTED_DATA:
-			error_message = g_strdup_printf (
-                        	_("Could not open the file \"%s\" because "
-			   	   "it contains corrupted data."),
-			 	uri_for_display);
-			break;			 
-
-		case GNOME_VFS_ERROR_NOT_SUPPORTED:
-			scheme_string = eel_uri_get_scheme (uri);
+		switch (error->code)
+		{
+			case GNOME_VFS_ERROR_NOT_FOUND:
+				error_message = g_strdup_printf (
+       	                 		_("Could not find the file \"%s\".\n\n"
+					  "Please, check that you typed the location correctly "
+					  "and try again."),
+       		                 	uri_for_display);
+				break;
+	
+			case GNOME_VFS_ERROR_CORRUPTED_DATA:
+				error_message = g_strdup_printf (
+       	                 		_("Could not open the file \"%s\" because "
+				   	   "it contains corrupted data."),
+				 	uri_for_display);
+				break;			 
+	
+			case GNOME_VFS_ERROR_NOT_SUPPORTED:
+				scheme_string = eel_uri_get_scheme (uri);
                 
-			if ((scheme_string != NULL) && g_utf8_validate (scheme_string, -1, NULL))
-			{
+				if ((scheme_string != NULL) && g_utf8_validate (scheme_string, -1, NULL))
+				{
+					error_message = g_strdup_printf (
+						_("Could not open the file \"%s\" because "
+						  "gedit cannot handle %s: locations."),
+       	                        	 	uri_for_display, scheme_string);
+				}
+				else
+					error_message = g_strdup_printf (
+						_("Could not open the file \"%s\""),
+       	                         		uri_for_display);
+			
+				if (scheme_string != NULL)
+					g_free (scheme_string);
+	
+       		 	        break;
+					
+			case GNOME_VFS_ERROR_WRONG_FORMAT:
+				error_message = g_strdup_printf (
+       		                 	_("Could not open the file \"%s\" because "
+				   	  "it contains data in an invalid format."),
+				 	uri_for_display);
+				break;	
+
+			case GNOME_VFS_ERROR_TOO_BIG:
+				error_message = g_strdup_printf (
+                        		_("Could not open the file \"%s\" because "
+			   		  "it is too big."),
+			 		uri_for_display);
+				break;	
+
+			case GNOME_VFS_ERROR_INVALID_URI:
+				error_message = g_strdup_printf (
+                	        	_("\"%s\" is not a valid location.\n\n"
+					  "Please, check that you typed the location "
+					  "correctly and try again."),
+                         		uri_for_display);
+	                	break;
+	
+			case GNOME_VFS_ERROR_ACCESS_DENIED:
 				error_message = g_strdup_printf (
 					_("Could not open the file \"%s\" because "
-					  "gedit cannot handle %s: locations."),
-                                	uri_for_display, scheme_string);
-			}
-			else
+					  "access was denied."),
+	                         	uri_for_display);
+        	        	break;
+
+			case GNOME_VFS_ERROR_TOO_MANY_OPEN_FILES:
 				error_message = g_strdup_printf (
-					_("Could not open the file \"%s\""),
-                                	uri_for_display);
-	
-			if (scheme_string != NULL)
-				g_free (scheme_string);
+					_("Could not open the file \"%s\" because "
+					  "there are too many open files.\n\n"
+					  "Please, close some open file and try again."),
+                	         	uri_for_display);
+                		break;
 
-        	        break;
-				
-		case GNOME_VFS_ERROR_WRONG_FORMAT:
-			error_message = g_strdup_printf (
-                        	_("Could not open the file \"%s\" because "
-			   	  "it contains data in an invalid format."),
-			 	uri_for_display);
-			break;	
+			case GNOME_VFS_ERROR_IS_DIRECTORY:
+				error_message = g_strdup_printf (
+					_("\"%s\" is a directory.\n\n"
+					  "Please, check that you typed the location "
+					  "correctly and try again."),
+                         		uri_for_display);
+	                	break;
 
-		case GNOME_VFS_ERROR_TOO_BIG:
-			error_message = g_strdup_printf (
-                        	_("Could not open the file \"%s\" because "
-			   	   "it is too big."),
-			 	uri_for_display);
-			break;	
+			case GNOME_VFS_ERROR_NO_MEMORY:
+				error_message = g_strdup_printf (
+					_("Not enough available memory to open the file \"%s\". "
+					  "Please, close some running application and try again."),
+	                         	uri_for_display);
+        	        	break;
 
-		case GNOME_VFS_ERROR_INVALID_URI:
-			error_message = g_strdup_printf (
-                        	_("\"%s\" is not a valid location.\n\n"
-				   "Please, check that you typed the location correctly and try again."),
-                         	uri_for_display);
-                	break;
-	
-		case GNOME_VFS_ERROR_ACCESS_DENIED:
-			error_message = g_strdup_printf (
-				_("Could not open the file \"%s\" because "
-				  "access was denied."),
-                         	uri_for_display);
-                	break;
-
-		case GNOME_VFS_ERROR_TOO_MANY_OPEN_FILES:
-			error_message = g_strdup_printf (
-				_("Could not open the file \"%s\" because "
-				  "there are too many open files.\n\n"
-				  "Please, close some open file and try again."),
-                         	uri_for_display);
-                	break;
-
-		case GNOME_VFS_ERROR_IS_DIRECTORY:
-			error_message = g_strdup_printf (
-				_("\"%s\" is a directory.\n\n"
-				  "Please, check that you typed the location correctly and try again."),
-                         	uri_for_display);
-                	break;
-
-		case GNOME_VFS_ERROR_NO_MEMORY:
-			error_message = g_strdup_printf (
-				_("Not enough available memory to open the file \"%s\". "
-				  "Please, close some running application and try again."),
-                         	uri_for_display);
-                	break;
-
-		case GNOME_VFS_ERROR_HOST_NOT_FOUND:
-			/* This case can be hit for user-typed strings like "foo" due to
-		 	* the code that guesses web addresses when there's no initial "/".
-		 	* But this case is also hit for legitimate web addresses when
-		 	* the proxy is set up wrong.
-		 	*/
-			{
-				gchar *host_name;
-				vfs_uri = gnome_vfs_uri_new (uri);
-				
-				if (vfs_uri == NULL)
-					host_name = g_strdup ("XXX");
-				else
+			case GNOME_VFS_ERROR_HOST_NOT_FOUND:
+				/* This case can be hit for user-typed strings like "foo" due to
+			 	* the code that guesses web addresses when there's no initial "/".
+			 	* But this case is also hit for legitimate web addresses when
+			 	* the proxy is set up wrong.
+		 		*/
 				{
-					host_name = eel_make_valid_utf8 (
+					gchar *host_name;
+					vfs_uri = gnome_vfs_uri_new (uri);
+				
+					if (vfs_uri == NULL)
+						host_name = g_strdup ("XXX");
+					else
+					{
+						host_name = eel_make_valid_utf8 (
 							gnome_vfs_uri_get_host_name (vfs_uri));
 
-					gnome_vfs_uri_unref (vfs_uri);		
-				}
+						gnome_vfs_uri_unref (vfs_uri);		
+					}
 				
-                		error_message = g_strdup_printf (
-					_("Could not open the file \"%s\" because no host \"%s\" " 
-				  	  "could be found.\n\n"
-                		  	  "Please, check that you typed the location correctly "
-				  	  "and that your proxy settings are correct and then "
-				  	  "try again."),
-					  uri_for_display,
-					  host_name);
+	                		error_message = g_strdup_printf (
+						_("Could not open the file \"%s\" because no host \"%s\" " 
+					  	  "could be found.\n\n"
+                			  	  "Please, check that you typed the location correctly "
+					  	  "and that your proxy settings are correct and then "
+				  		  "try again."),
+						  uri_for_display,
+						  host_name);
 
-				g_free (host_name);
-			}
+					g_free (host_name);
+				}
 
-			break;
+				break;
 
-		case GNOME_VFS_ERROR_INVALID_HOST_NAME:
-			error_message = g_strdup_printf (
-                        	_("Could not open the file \"%s\" because the host name was invalid.\n\n"
-				   "Please, check that you typed the location correctly and try again."),
-                         	uri_for_display);
-                	break;
+			case GNOME_VFS_ERROR_INVALID_HOST_NAME:
+				error_message = g_strdup_printf (
+                        		_("Could not open the file \"%s\" because the host name "
+					  "was invalid.\n\n"
+				   	  "Please, check that you typed the location correctly "
+					  "and try again."),
+	                         	uri_for_display);
+                		break;
 
-		case GNOME_VFS_ERROR_HOST_HAS_NO_ADDRESS:
-			error_message = g_strdup_printf (
-				_("Could not open the file \"%s\" because the host name was empty.\n\n"
-				  "Please, check that your proxy settings are correct and try again."),
-				uri_for_display);
-			break;
+			case GNOME_VFS_ERROR_HOST_HAS_NO_ADDRESS:
+				error_message = g_strdup_printf (
+					_("Could not open the file \"%s\" because the host name "
+					  "was empty.\n\n"
+					  "Please, check that your proxy settings are correct "
+					  "and try again."),
+					uri_for_display);
+				break;
 		
-		case GNOME_VFS_ERROR_LOGIN_FAILED:
-			error_message = g_strdup_printf (
-				_("Could not open the file \"%s\" because the attempt to "
-				  "log in failed.\n\n"
-				  "Please, check that you typed the location correctly and try again."),
-				uri_for_display);		
-			break;
+			case GNOME_VFS_ERROR_LOGIN_FAILED:
+				error_message = g_strdup_printf (
+					_("Could not open the file \"%s\" because the attempt to "
+					  "log in failed.\n\n"
+					  "Please, check that you typed the location correctly "
+					  "and try again."),
+					uri_for_display);		
+				break;
 
-		case GEDIT_ERROR_INVALID_UTF8_DATA:
-			error_message = g_strdup_printf (
-                        	_("Could not open the file \"%s\" because "
-			   	  "it contains invalid UTF-8 data.\n\n"
-				  "Probably, you are trying to open a binary file."),
-			 	uri_for_display);
+			case GEDIT_ERROR_INVALID_UTF8_DATA:
+				error_message = g_strdup_printf (
+                	        	_("Could not open the file \"%s\" because "
+				   	  "it contains invalid data.\n\n"
+				  	  "Probably, you are trying to open a binary file."),
+				 	uri_for_display);
 
-			break;
+				break;
 
-		/*
-		case GNOME_VFS_ERROR_GENERIC:
-		case GNOME_VFS_ERROR_INTERNAL:
-		case GNOME_VFS_ERROR_BAD_PARAMETERS:
-		case GNOME_VFS_ERROR_IO:
-		case GNOME_VFS_ERROR_BAD_FILE:
-		case GNOME_VFS_ERROR_NO_SPACE:
-		case GNOME_VFS_ERROR_READ_ONLY:
-		case GNOME_VFS_ERROR_NOT_OPEN:
-		case GNOME_VFS_ERROR_INVALID_OPEN_MODE:
-		case GNOME_VFS_ERROR_EOF:
-		case GNOME_VFS_ERROR_NOT_A_DIRECTORY:
-		case GNOME_VFS_ERROR_IN_PROGRESS:
-		case GNOME_VFS_ERROR_INTERRUPTED:
-		case GNOME_VFS_ERROR_FILE_EXISTS:
-		case GNOME_VFS_ERROR_LOOP:
-		case GNOME_VFS_ERROR_NOT_PERMITTED:
-		case GNOME_VFS_ERROR_CANCELLED:
-		case GNOME_VFS_ERROR_DIRECTORY_BUSY:
-		case GNOME_VFS_ERROR_DIRECTORY_NOT_EMPTY:
-		case GNOME_VFS_ERROR_TOO_MANY_LINKS:
-		case GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM:
-		case GNOME_VFS_ERROR_NOT_SAME_FILE_SYSTEM:
-		case GNOME_VFS_ERROR_NAME_TOO_LONG:
-		case GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE:
-		case GNOME_VFS_ERROR_SERVICE_OBSOLETE,
-		case GNOME_VFS_ERROR_PROTOCOL_ERROR,
-		case GNOME_VFS_NUM_ERRORS:
-		*/
-		default:
-			error_message = g_strdup_printf (
-                        	_("Could not open the file \"%s\"."),
-			 	uri_for_display);
+			/*
+			case GNOME_VFS_ERROR_GENERIC:
+			case GNOME_VFS_ERROR_INTERNAL:
+			case GNOME_VFS_ERROR_BAD_PARAMETERS:
+			case GNOME_VFS_ERROR_IO:
+			case GNOME_VFS_ERROR_BAD_FILE:
+			case GNOME_VFS_ERROR_NO_SPACE:
+			case GNOME_VFS_ERROR_READ_ONLY:
+			case GNOME_VFS_ERROR_NOT_OPEN:
+			case GNOME_VFS_ERROR_INVALID_OPEN_MODE:
+			case GNOME_VFS_ERROR_EOF:
+			case GNOME_VFS_ERROR_NOT_A_DIRECTORY:
+			case GNOME_VFS_ERROR_IN_PROGRESS:
+			case GNOME_VFS_ERROR_INTERRUPTED:
+			case GNOME_VFS_ERROR_FILE_EXISTS:
+			case GNOME_VFS_ERROR_LOOP:
+			case GNOME_VFS_ERROR_NOT_PERMITTED:
+			case GNOME_VFS_ERROR_CANCELLED:
+			case GNOME_VFS_ERROR_DIRECTORY_BUSY:
+			case GNOME_VFS_ERROR_DIRECTORY_NOT_EMPTY:
+			case GNOME_VFS_ERROR_TOO_MANY_LINKS:
+			case GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM:
+			case GNOME_VFS_ERROR_NOT_SAME_FILE_SYSTEM:
+			case GNOME_VFS_ERROR_NAME_TOO_LONG:
+			case GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE:
+			case GNOME_VFS_ERROR_SERVICE_OBSOLETE,
+			case GNOME_VFS_ERROR_PROTOCOL_ERROR,
+			case GNOME_VFS_NUM_ERRORS:
+			*/
+		}
+	}
+	else if (error->domain == GEDIT_CONVERT_ERROR)
+	{
+		switch (error->code)
+		{
+			case GEDIT_CONVERT_ERROR_AUTO_DETECTION_FAILED:
+				error_message = g_strdup_printf (
+                	        	_("Could not open the file \"%s\" because "
+				   	  "gedit has not been able to automatically detect "
+					  "the character coding.\n\n"
+					  "Please, check that you are not trying to open a binary file "
+					  "and try again selecting a character coding in the 'Open File...' "
+					  "(or 'Open Location') dialog."),
+				 	uri_for_display);
 
-			break;
+				break;
+
+			case GEDIT_CONVERT_ERROR_ILLEGAL_SEQUENCE:
+				error_message = g_strdup_printf (
+                	        	_("Could not open the file \"%s\" using "
+					  "the %s character coding.\n\n"
+					  "Please, check that you are not trying to open a binary file "
+					  "and that you selected the right "
+					  "character coding in the 'Open File...' (or 'Open Location') "
+					  "dialog and try again."),
+					uri_for_display, encoding_name);
+
+				break;
+			
+			case GEDIT_CONVERT_ERROR_BINARY_FILE:
+				error_message = g_strdup_printf (
+                	        	_("Could not open the file \"%s\" because "
+				   	  "it contains invalid data.\n\n"
+				  	  "Probably, you are trying to open a binary file."),
+				 	uri_for_display);
+
+				break;
+	  
+		}
+	}
+	else if (error->domain == G_CONVERT_ERROR)
+	{
+		switch (error->code)
+		{
+			case G_CONVERT_ERROR_NO_CONVERSION:
+			case G_CONVERT_ERROR_ILLEGAL_SEQUENCE:
+			case G_CONVERT_ERROR_FAILED:
+			case G_CONVERT_ERROR_PARTIAL_INPUT:
+				error_message = g_strdup_printf (
+					_("Could not open the file \"%s\" using "
+					  "the %s character coding.\n\n"
+					  "Please, check that you are not trying to open a binary file "
+					  "and that you selected the right "
+					  "character coding in the 'Open File...' (or 'Open Location') "
+					  "dialog and try again."),
+					uri_for_display, encoding_name);
+				break;
+
+		}
 	}
 	
+	if (error_message == NULL)
+	{
+		if (error->message == NULL)
+			error_message = g_strdup_printf (
+                        			_("Could not open the file \"%s\"."),
+					 	uri_for_display);
+		else
+			error_message = g_strdup_printf (
+                        			_("Could not open the file \"%s\".\n\n%s"),
+					 	uri_for_display, error->message);
+
+	}
+
+	g_free (encoding_name);
+
 	dialog = gtk_message_dialog_new (
 			parent,
 			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -1590,6 +1284,25 @@ gedit_utils_get_stdin (void)
 	return g_string_free (file_contents, FALSE);
 }
 
+void 
+gedit_warning (gchar *str, GtkWindow *parent)
+{
+	GtkWidget *dialog;
 
+	dialog = gtk_message_dialog_new (
+			parent,
+			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+		   	GTK_MESSAGE_ERROR,
+		   	GTK_BUTTONS_OK,
+			str);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
+	gtk_dialog_run (GTK_DIALOG (dialog));
+
+	gtk_widget_destroy (dialog);
+}
 
 
