@@ -36,8 +36,7 @@
 #include <gtk/gtkdialog.h>
 #include <gtk/gtktextiter.h>
 #include <atk/atk.h>
-
-#define GTK_TEXT_SEARCH_CASE_INSENSITIVE (1 << 7)
+#include <gedit/gedit-encodings.h>
 
 void	gedit_utils_flash     (gchar *msg);
 void	gedit_utils_flash_va  (gchar *format, ...);
@@ -53,17 +52,11 @@ GtkWidget *gedit_dialog_add_button (GtkDialog *dialog,
 				    const gchar* stock_id, 
 				    gint response_id);
 
-gboolean gedit_text_iter_forward_search  (const GtkTextIter *iter,
-					  const gchar       *str,
-					  GtkTextSearchFlags flags,
-					  GtkTextIter       *match_start,
-					  GtkTextIter       *match_end,
-					  const GtkTextIter *limit);
-
 gchar *gedit_utils_str_middle_truncate (const gchar *string, 
 					guint truncate_length);
 
 void gedit_utils_error_reporting_loading_file (const gchar *uri, 
+					       const GeditEncoding *encoding,
 					       GError *error,
 					       GtkWindow *parent);
 void gedit_utils_error_reporting_saving_file  (const gchar *uri, 
@@ -91,6 +84,8 @@ gboolean gedit_utils_create_empty_file (const gchar *uri);
 
 gchar *gedit_utils_get_stdin (void);
 
+void gedit_warning (gchar *str, GtkWindow *parent);
+
 #endif /* __GEDIT_UTILS_H__ */
 
 
